#**************************************************************************************
#********************************* gid raised events **********************************
proc InitGIDProject { dir } {
    set HIRF::_dir $dir  
    if { [info procs ReadProblemtypeXml] != "" } {
        #this procedure exists after GiD 11.1.2b
        set data [ReadProblemtypeXml [file join $dir HIRF.xml] Infoproblemtype {Version MinimumGiDVersion MinimumAmeletPluginVersion}]                
    } else {
        #this procedure is a copy of ReadProblemtypeXml to be able to work with previous GiD's
        set data [HIRF::ReadProblemtypeXml [file join $dir HIRF.xml] Infoproblemtype {Version MinimumGiDVersion MinimumAmeletPluginVersion}]
    }    
    if { $data == "" } {
        WarnWinText [= "Configuration file %s not found" [file join $dir HIRF.xml]]
        return 1
    }
    array set problemtype_local $data
    set HIRF::VersionNumber $problemtype_local(Version)
       
    if { [GidUtils::VersionCmp $problemtype_local(MinimumGiDVersion)] < 0 } {  
        WarnWinText [= "This problemtype requires GiD %s or later" $problemtype_local(MinimumGiDVersion)]
    }
    if { ![info exists ::Amelet::version] || $::Amelet::version < $problemtype_local(MinimumAmeletPluginVersion) } {  
        WarnWinText [= "This problemtype requires Amelet plugin %s or later" $problemtype_local(MinimumAmeletPluginVersion)]
    }
    HIRF::ModifyMenu
}

proc EndGIDProject {} {     
    HIRF::DeleteEvents
    HIRF::RestoreMenu  
}

proc ChangedLanguage { newlan } {
    HIRF::ModifyMenu
}

proc AfterMeshGeneration { fail } {
    if { !$fail } {
        HIRF::TransferSpecialConditions
    }
    return ""
}

proc LoadGIDProject { filespd } {
    if { [file join {*}[lrange [file split $filespd] end-1 end]] == "HIRF.gid/HIRF.spd" } {
        #loading the problemtype itself, not a model
    } else {
        set pt [GiD_Info project ProblemType]
        if { $pt == "HIRF" } {
            set filename [file rootname $filespd].xml
            set model_problemtype_version_number [HIRF::ReadXml $filename]            
            if { $model_problemtype_version_number != -1 && $model_problemtype_version_number < $::HIRF::VersionNumber } {            
                set must_transform 1
            } else {
                set must_transform 0
            }
            if { $must_transform } {
                after idle [list HIRF::Transform $model_problemtype_version_number $::HIRF::VersionNumber]
            }
        }
    }
}

proc SaveGIDProject { filespd } {
    set filename [file rootname $filespd].xml
    HIRF::SaveXml $filename
}
 
#**************************************************************************************
#********************************* problemtype auxiliary procedures *******************
namespace eval HIRF {
    variable ProgramName HIRF
    variable VersionNumber ;#interface version, get it from xml to avoid duplication 
    variable _dir ;#path to the problemtype
}

proc HIRF::SaveXml { filename } {
    variable ProgramName
    variable VersionNumber
    set fp [open $filename w]
    if { $fp != "" } {
        puts $fp {<?xml version='1.0' encoding='utf-8'?><!-- -*- coding: utf-8;-*- -->}
        puts $fp "<$ProgramName version='$VersionNumber'/>"
        close $fp
    }
}

#return -1 if the xml is not an HIRF one or if the xml not exists
#        the HIRF version of the model of the xml file
proc HIRF::ReadXml { filename } {
    variable ProgramName
    set model_problemtype_version_number -1
    if { [file exists $filename] } {
        set fp [open $filename r]
        if { $fp != "" } {
            set line ""
            gets $fp header
            gets $fp line ;#something like: <HIRF version='1.0'/>
            close $fp
            set line [string range $line 1 end-2]
            set model_problemtype [lindex $line 0]
            if { $model_problemtype == $ProgramName } {
                set model_version [lindex $line 1]
                if { [lindex [split $model_version =] 0] == "version" } {
                    set model_problemtype_version_number [string range [lindex [split $model_version =] 1] 1 end-1]
                }
            } else {
                set model_problemtype_version_number -1
            }
        }
    }
    return $model_problemtype_version_number
}

proc HIRF::Transform { from_version to_version } {    
    variable ProgramName
    GiD_Process escape escape escape escape Data Defaults TransfProblem $ProgramName escape    
}

#this procedure is a copy of ReadProblemtypeXml to be able to work with previous GiD's
proc HIRF::ReadProblemtypeXml { xmlfile root_name fields } {
    set res {}
    if { [file exists $xmlfile] && [file isfile $xmlfile]} {
        package require tdom    
        set xml [tDOM::xmlReadFile $xmlfile]
        set doc [dom parse $xml]
        set root [$doc documentElement]  
        if { $root_name == "" || [$root nodeName] == $root_name} {
            #set file_version [$root getAttribute version]
            if { $fields == "" } {
                set fields ""
                foreach node [$root selectNodes Program/*] {
                    lappend fields [$node nodeName]
                }
            } 
            foreach field $fields {
                set node [$root select Program/$field]
                if {$node ne ""} {
                    set value [$node text]
                } else {
                    set value ""
                }     
                lappend res $field $value
            }
        }
        $doc delete
    }
    return $res
}

proc HIRF::ModifyMenu { } {
    variable ProgramName
    GiDMenu::Create HIRF PRE
    GiDMenu::InsertOption HIRF Collisions 0 PRE HIRF::GetLineElementIntersectingQuadrilaterals 
    ::GiDMenu::InsertOption HIRF [list [concat [= "About"] $ProgramName ...]] 1 PRE HIRF::About "" "" replace =
    GiDMenu::UpdateMenus
}

proc HIRF::RestoreMenu { } {
    GiDMenu::Delete HIRF PRE
    GiDMenu::UpdateMenus
}

proc HIRF::Splash { {self_close 1} } {
    variable _dir
    set prev_splash_state [GiD_Set SplashWindow]
    GiD_Set SplashWindow 1 ;#set temporary to 1 to force show splash without take care of the GiD splash preference
    set txt "$HIRF::ProgramName Version $HIRF::VersionNumber"
    append txt \n\n[= "Interface developed by: %s\n" "CIMNE"]
    ::GidUtils::Splash [file join $HIRF::_dir images splash.png] .splash $self_close [list $txt 150 250]
    GiD_Set SplashWindow $prev_splash_state
}

proc HIRF::About { } {
    set self_close 0
    HIRF::Splash $self_close
}   

proc HIRF::GetClosestLineElement { xyz line_elements } {
    set min_element_id ""
    #calculate a 'eliptical distance squared' point-edge
    set d2min 1e20
    foreach element_id $line_elements {
        set data [GiD_Mesh get element $element_id]
        set element_connectivities [lrange $data 3 4]
        set cont 0
        foreach node_id $element_connectivities {
            set data [GiD_Mesh get node $node_id]
            set element_vertex($cont) [lrange $data 1 end]
            incr cont
        }
        set va [math::linearalgebra::sub_vect $xyz $element_vertex(0)]
        set da2 [math::linearalgebra::dotproduct $va $va]
        set vb [math::linearalgebra::sub_vect $xyz $element_vertex(1)]
        set db2 [math::linearalgebra::dotproduct $vb $vb]
        set vab [math::linearalgebra::sub_vect $element_vertex(0) $element_vertex(1)]
        set dab2 [math::linearalgebra::dotproduct $vab $vab]
        set d2 [expr {$da2+$db2-$dab2}]
        if { $d2min > $d2 } {
            set d2min $d2
            set min_element_id $element_id
        }
    }
    return $min_element_id
}

proc HIRF::GetClosestElement { element_level xyz element_ids } {
    set min_element_id ""
    set d2_min 1e20
    foreach element_id $element_ids {        
        set res [HIRF::GetElementRelativeCoordinatesFromCoord $element_level $xyz $element_id]        
        if { $element_level == "line" } {
            lassign $res t d2
            if { $t>=0.0 && $t<=1.0 } {   
                if { $d2_min > $d2 } {
                    set d2_min $d2             
                    set min_element_id $element_id
                }
            }
        } elseif { $element_level == "surface" } {
            lassign $res u v d2
            if { $u>=0.0 && $u<=1.0 && $v>=0.0 && $v<=1.0  } {
                if { $d2_min > $d2 } {
                    set d2_min $d2  
                    set min_element_id $element_id
                }
            }
        } elseif { $element_level == "volume" } {
            lassign $res u v w d2
            if { $u>=0.0 && $u<=1.0 && $v>=0.0 && $v<=1.0  && $w>=0.0 && $w<=1.0 } {
                if { $d2_min > $d2 } {
                    set d2_min $d2
                    set min_element_id $element_id
                }
            }            
        }
    }
    return $min_element_id
}

proc HIRF::GetClosestLine { xyz } {    
    set min_line_id ""
    set d2min 1e20
    foreach line_id [GiD_Geometry  list line 1:end] {
        set t [GiD_Info Parametric Line $line_id t_fromcoord {*}$xyz]
        set xyz2 [GiD_Info Parametric Line $line_id coord $t]
        set v [math::linearalgebra::sub_vect $xyz $xyz2]
        set d2 [math::linearalgebra::dotproduct $v $v]
        if { $d2min > $d2 } {
            set d2min $d2
            set min_line_id $line_id
            if { $d2min < 1e-10 } {
                break
            }
        }
    }           
    set line_id $min_line_id         
}

proc HIRF::TransferSpecialConditions { } {
    #Line_Point_in_element, Surface_Point_in_element and  Volume_Point_in_element
    #must replace all elements of the same geometric entiy to a single element where the parameters lies
    #warning: must do this trick before the trick of Point_To_Line_Point_in_element one 
    #because it also apply new conditions Line_Point_in_element to elements
    set condition_names {Line_Point_in_element Surface_Point_in_element Volume_Point_in_element}
    foreach condition_name $condition_names element_level {line surface volume} {
        array unset geometry_elements
        foreach item [GiD_Info Conditions $condition_name mesh] {
            set element_id [lindex $item 1]
            #use as key all applied values instead geometry_id, because allow repeat the condition, then the geometry_id is not enough
            #the values are then stored in the key
            set key [lrange $item 3 end]
            
            lappend geometry_elements($key) $element_id
        }
        GiD_UnAssignData Condition $condition_name Elements all
        foreach key [array names geometry_elements] {
            lassign $key name parameters geometry_id ;#Id hidden field to identify geometry parent
            set xyz [GiD_Info Parametric $element_level $geometry_id coord {*}$parameters]   
            set element_id [HIRF::GetClosestElement $element_level $xyz $geometry_elements($key)]
            if { $element_id != "" } {
                set res [HIRF::GetElementRelativeCoordinatesFromCoord $element_level $xyz $element_id]  
                set relative_coordinates [lrange $res 0 end-1]
                set new_values [list $name $relative_coordinates $geometry_id]
                GiD_AssignData condition $condition_name Elements $new_values $element_id
            } else {
                WarnWinText "projection $element_level element not found"
            }
        }
    }
    

    
    #Point_To_Line_Point_in_element
    foreach item [GiD_Info Conditions Point_To_Line_Point_in_element mesh] {
        set node_id [lindex $item 1]
        set value [lindex $item 3]
        lappend node_values($node_id) $value
    }
    set nodes_condition [lsort -integer [array names node_values]]
    set data [lindex [GiD_Info Mesh Elements Linear -array] 0]        
    foreach element_id [lindex $data 1] n0 [lindex [lindex $data 2] 0] n1 [lindex [lindex $data 2] 1] {
        if { [lsearch -integer -sorted $nodes_condition $n0] != -1 } {
            foreach name $node_values($n0) {
                GiD_AssignData condition Line_Point_in_element Elements [list $name 0.0 0] $element_id
            }
            incr nodes_used($n0)
        } elseif { [lsearch -integer -sorted $nodes_condition $n1] != -1 } {
            foreach name $node_values($n1) {
                GiD_AssignData condition Line_Point_in_element Elements [list $name 1.0 0] $element_id
            }
            incr nodes_used($n1)
        }
    }
    GiD_UnAssignData Condition Point_To_Line_Point_in_element Nodes all

    foreach node_id $nodes_condition {
        if { ![info exists nodes_used($node_id)] } {
            WarnWinText [= "Lost condition 'Point_To_Line_Point_in_element' on node %s with value %s" $node_id $node_values($node_id)]
        }
    }
    
}

#delete event procedures to avoid to be invoked by Amelet plugin when problemtype is unloaded
proc HIRF::DeleteEvents { } {
    set events {
        Amelet::ReadPreSelectorOnmeshRecursivelly 
        Amelet::WritePreSelectorOnmeshUnstructuredMesh  
        Amelet::WritePreSelectorOnmeshStructuredMesh
    }
    foreach name $events {
        rename $name {}
    }
} 

#*************auxiliary function to detect intersecting grid lines and squares *********
proc HIRF::GetLineElementIntersectingQuadrilaterals { } {
    lassign [lindex [GiD_Info Mesh Elements Quadrilateral -array] 0] element_type element_ids element_connectivities element_materials   
    lassign $element_connectivities nodes_0 nodes_1 nodes_2 nodes_3
    foreach element_id $element_ids node_0 $nodes_0 node_1 $nodes_1 node_2 $nodes_2 node_3 $nodes_3 {    
        foreach node_id [list $node_0 $node_1 $node_2 $node_3] {
            lappend quadrilateral_node($node_id) $element_id
        }
    }
    
    lassign [lindex [GiD_Info Mesh Elements Linear -array] 0] element_type element_ids element_connectivities element_materials
    lassign $element_connectivities nodes_0 nodes_1
    set joined_lines_1 [list ]
    set joined_lines_2 [list ]
    set joined_nodes [list ]
    foreach element_id $element_ids node_0 $nodes_0 node_1 $nodes_1 {
        set joined 0
        foreach node_id [list $node_0 $node_1] {
            if { [info exists quadrilateral_node($node_id)] } {
                incr joined
                lappend joined_nodes $node_id
            }
        }        
        if { $joined == 1 } {
            lappend joined_lines_1 $element_id
        } elseif { $joined == 2 } {
            lappend joined_lines_2 $element_id
        }
    }

    #remove repeateds
    set joined_nodes [lsort -integer -unique $joined_nodes]

    if { [llength $joined_nodes] == 0 } {
        WarnWinText "No collision"
    } else {
        set joined_quadrilaterals [list ]
        foreach node_id $joined_nodes {
            lappend joined_quadrilaterals {*}$quadrilateral_node($node_id)
        }
        #remove repeateds
        set joined_quadrilaterals [lsort -integer -unique $joined_quadrilaterals]

        set show_message 0
        if { $show_message } {
            WarnWinText "Nodes colliding:"
            WarnWinText $joined_nodes
            WarnWinText "Line elements touching 1 quadrilateral:"
            WarnWinText $joined_lines_1
            WarnWinText "Line elements touching 2 quadrilaterals:"
            WarnWinText $joined_lines_2
            WarnWinText "Cuadrilateral elements colliding:"
            WarnWinText $joined_quadrilaterals
        }
        set send_to_layers 1
        if { $send_to_layers } {
            set layers [GiD_Info Layers]            
            foreach layer_name {Collision Collision//Nodes Collision//Lines_1 Collision//Lines_2 Collision//Quadrilaterals } {
                if { [lsearch $layers $layer_name] == -1 } {
                    GiD_Process 'Layers New $layer_name escape
                }
            }
            GidUtils::DisableGraphics
            GiD_Process 'Layers Entities Collision//Nodes Nodes {*}$joined_nodes escape
            GiD_Process 'Layers Entities Collision//Lines_1 LowerEntities Elements {*}$joined_lines_1 escape
            GiD_Process 'Layers Entities Collision//Lines_2 LowerEntities Elements {*}$joined_lines_2 escape
            GiD_Process 'Layers Entities Collision//Quadrilaterals LowerEntities Elements {*}$joined_quadrilaterals escape
            GiD_Process 'Layers Entities Collision//Nodes Nodes {*}$joined_nodes escape
            GidUtils::EnableGraphics
            WarnWinText "Colliding entities sent to Layer 'Collision'"
        }
    }
    return
}


#to be used by TKWIDGET to easily add a pick node button
#e.g.
#QUESTION: your_question
#VALUE: your_value
#TKWIDGET: HIRF::TkwidgetPickPointInElement
proc HIRF::TkwidgetPickPointInElement { event args } {
    global tkwidgedprivpicknodebuttons
    switch $event {
        INIT {
            set PARENT [lindex $args 0]
            upvar [lindex $args 1] ROW
            set GDN [lindex $args 2]
            set STRUCT [lindex $args 3]
            set QUESTION [lindex $args 4]
            set entry ""
            foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
                if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
                    #assumed that it is the only entry of this row
                    set entry $item
                    break
                }
            }            
            if { $STRUCT == "CND,Line_Point_in_element" } {
                set element_level line
            } elseif { $STRUCT == "CND,Surface_Point_in_element" } {
                set element_level surface
            } elseif { $STRUCT == "CND,Volume_Point_in_element" } {
                set element_level volume
            } else {
                set element_level ""
            }
            
            if { [GiD_Info project ViewMode] == "GEOMETRYUSE" }  {
                if { $element_level == "line" } {
                    set tkwidgedprivpicknodebuttons($QUESTION) [button $PARENT.bpicknode$QUESTION \
                            -image  [gid_themes::GetImage "point.png" small_icons] \
                            -command [list HIRF::_PickPointInLineCmd $entry $element_level]]
                } elseif { $element_level == "surface" } {
                    set tkwidgedprivpicknodebuttons($QUESTION) [button $PARENT.bpicknode$QUESTION \
                            -image  [gid_themes::GetImage "point.png" small_icons] \
                            -command [list HIRF::_PickPointInSurfaceCmd $entry $element_level]]
                } elseif { $element_level == "volume" } {
                    set tkwidgedprivpicknodebuttons($QUESTION) [button $PARENT.bpicknode$QUESTION \
                            -image  [gid_themes::GetImage "point.png" small_icons] \
                            -command [list HIRF::_PickPointInVolumeCmd $entry $element_level]]
                } else {
                }                
            } elseif { [GiD_Info project ViewMode] == "MESHUSE" }  {
                set tkwidgedprivpicknodebuttons($QUESTION) [button $PARENT.bpicknode$QUESTION \
                        -image  [gid_themes::GetImage "point.png" small_icons] \
                        -command [list HIRF::_PickPointInElementCmd $entry $element_level]]
            }
            grid $tkwidgedprivpicknodebuttons($QUESTION) -row [expr $ROW-1] -column 2 -sticky w
            grid configure $entry -sticky ew
            return ""
        }
        SYNC {
            #set GDN [lindex $args 0]
            #set STRUCT [lindex $args 1]
            #set QUESTION [lindex $args 2]
            #DWLocalSetValue $GDN $STRUCT $QUESTION $value
        }
        DEPEND {
            #set GDN [lindex $args 0]
            #set STRUCT [lindex $args 1]
            set QUESTION [lindex $args 2]
            set ACTION [lindex $args 3]
            #set value [lindex $args 4]
            if { [info exists tkwidgedprivpicknodebuttons($QUESTION)] && \
                    [winfo exists $tkwidgedprivpicknodebuttons($QUESTION)] } {
                if { $ACTION == "HIDE" } {
                    grid remove $tkwidgedprivpicknodebuttons($QUESTION)
                } else {
                    #RESTORE
                    grid $tkwidgedprivpicknodebuttons($QUESTION)
                }
            }
        }
        CLOSE {
            array unset tkwidgedprivpicknodebuttons
        }
        default {
            return [list ERROR [_ "Unexpected tkwidget event"]]
        }
    }
}

#axiliary procedure used by HIRF::TkwidgetPickPointInElement
package require math::linearalgebra

proc HIRF::_PickPointInLineCmd { entry element_level } {    
    set xyz [GidUtils::GetCoordinates [_ "Pick point into an curve"] PointInLine GEOMETRYUSE]
    if { $xyz != "" } {
        if { [GidUtils::VersionCmp 11.1.3d] >= 0 } {
            #older versions not provide the line_in in GidPriv(selection)
            set line_id $::GidPriv(selection)
        } else {            
            set line_id [HIRF::GetClosestLine $xyz]            
        }
        if { $line_id != "" } {
            set t [GiD_Info Parametric Line $line_id t_fromcoord {*}$xyz]       
            set res [format %f $t]
            $entry delete 0 end
            $entry insert end $res        
        }
    }
}

proc HIRF::_PickPointInSurfaceCmd { entry element_level } {    
    set xyz [GidUtils::GetCoordinates [_ "Pick point into an surface"] PointInSurface GEOMETRYUSE]
    if { $xyz != "" } {
        set surface_id $::GidPriv(selection)        
        if { $surface_id != "" } {
            set uv [GiD_Info Parametric Surface $surface_id uv_fromcoord {*}$xyz]       
            lassign $uv u v
            set res [list [format %f $u] [format %f $v]]
            $entry delete 0 end
            $entry insert end $res        
        }
    }
}

#return the relative coordinates and the distance2 from xyz to the projected point
#but the projection could lie outside the segment, triangle or tetrahedron
proc HIRF::GetElementRelativeCoordinatesFromCoord { element_level xyz element_id } {          
    set data [GiD_Mesh get element $element_id]
    set element_type [lindex $data 1]
    set element_nnodes [lindex $data 2]        
    if { $element_type == "Linear" } {
        if { $element_level != "line" } {
            WarnWin [_ "A %s element must be clicked" $element_level]
            return
        }
        set element_connectivities [lrange $data 3 4]
        set cont 0
        foreach node_id $element_connectivities {
            set data [GiD_Mesh get node $node_id]
            set element_vertex($cont) [lrange $data 1 end]
            incr cont
        }
        set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
        set op [math::linearalgebra::sub_vect $xyz $element_vertex(0)]
        
        set v11 [math::linearalgebra::dotproduct $v1 $v1]            
        set v1p [math::linearalgebra::dotproduct $v1 $op]      
        set u [expr {$v1p/$v11}]
        set op_projected [math::linearalgebra::scale_vect $u $v1]
        set op_normal [math::linearalgebra::sub_vect $op $op_projected]
        set d2 [math::linearalgebra::dotproduct $op_normal $op_normal]
        set res [format "%f %f" $u $d2]
    } elseif { $element_type == "Triangle" || $element_type == "Quadrilateral"} {
        if { $element_level != "surface" } {
            WarnWin [_ "A %s element must be clicked" $element_level]
            return
        }
        if { $element_type == "Triangle"} {
            set element_connectivities [lrange $data 3 5]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }                
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(2) $element_vertex(0)]
        } elseif { $element_type == "Quadrilateral"} {
            set element_connectivities [lrange $data 3 6]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }                
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(3) $element_vertex(0)]
        }
        
        set op [math::linearalgebra::sub_vect $xyz $element_vertex(0)]
        
        set v11 [math::linearalgebra::dotproduct $v1 $v1]
        set v12 [math::linearalgebra::dotproduct $v1 $v2]
        set v22 [math::linearalgebra::dotproduct $v2 $v2]
        set matrix [::math::linearalgebra::mkMatrix 2 2]            
        math::linearalgebra::setrow matrix 0 [list $v11 $v12]
        math::linearalgebra::setrow matrix 1 [list $v12 $v22]
        
        set v1p [math::linearalgebra::dotproduct $v1 $op]
        set v2p [math::linearalgebra::dotproduct $v2 $op]
        set bvect [::math::linearalgebra::mkVector 2]
        math::linearalgebra::setcol bvect 0 [list $v1p $v2p]
        
        lassign [::math::linearalgebra::solveGauss $matrix $bvect] u v
      
        set op_projected [math::linearalgebra::scale_vect $u $v1]
        set op_projected [math::linearalgebra::add $op_projected [math::linearalgebra::scale_vect $v $v2]]
        set op_normal [math::linearalgebra::sub_vect $op $op_projected]
        set d2 [math::linearalgebra::dotproduct $op_normal $op_normal]
        set res [format "%f %f %f" $u $v $d2]
    } elseif { $element_type == "Tetrahedra" || $element_type == "Hexahedra" || $element_type == "Prism" || $element_type == "Pyramid"} {
        if { $element_level != "volume" } {
            WarnWin [_ "A %s element must be clicked" $element_level]
            return
        }
        if { $element_type == "Tetrahedra"} {
            set element_connectivities [lrange $data 3 6]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }            
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(2) $element_vertex(0)]
            set v3 [math::linearalgebra::sub_vect $element_vertex(3) $element_vertex(0)]
        } elseif { $element_type == "Hexahedra"} {
            set element_connectivities [lrange $data 3 10]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }            
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(3) $element_vertex(0)]
            set v3 [math::linearalgebra::sub_vect $element_vertex(4) $element_vertex(0)]
        } elseif { $element_type == "Prism"} {
            set element_connectivities [lrange $data 3 8]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }            
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(3) $element_vertex(0)]
            set v3 [math::linearalgebra::sub_vect $element_vertex(2) $element_vertex(0)]
        } elseif { $element_type == "Pyramid"} {
            set element_connectivities [lrange $data 3 7]
            set cont 0
            foreach node_id $element_connectivities {
                set data [GiD_Mesh get node $node_id]
                set element_vertex($cont) [::math::linearalgebra::mkVector 3]
                math::linearalgebra::setcol element_vertex($cont) 0 [lrange $data 1 end]
                incr cont
            }            
            set v1 [math::linearalgebra::sub_vect $element_vertex(1) $element_vertex(0)]
            set v2 [math::linearalgebra::sub_vect $element_vertex(3) $element_vertex(0)]
            set v3 [math::linearalgebra::sub_vect $element_vertex(4) $element_vertex(0)]
        }
        
        set op [math::linearalgebra::sub_vect $xyz $element_vertex(0)]
        
        set v11 [math::linearalgebra::dotproduct $v1 $v1]
        set v12 [math::linearalgebra::dotproduct $v1 $v2]
        set v13 [math::linearalgebra::dotproduct $v1 $v3]
        set v22 [math::linearalgebra::dotproduct $v2 $v2]
        set v23 [math::linearalgebra::dotproduct $v2 $v3]
        set v33 [math::linearalgebra::dotproduct $v3 $v3]
        set matrix [::math::linearalgebra::mkMatrix 3 3]            
        math::linearalgebra::setrow matrix 0 [list $v11 $v12 $v13]
        math::linearalgebra::setrow matrix 1 [list $v12 $v22 $v23]
        math::linearalgebra::setrow matrix 2 [list $v13 $v23 $v33]
        
        set v1p [math::linearalgebra::dotproduct $v1 $op]
        set v2p [math::linearalgebra::dotproduct $v2 $op]
        set v3p [math::linearalgebra::dotproduct $v3 $op]
        set bvect [::math::linearalgebra::mkVector 3]
        math::linearalgebra::setcol bvect 0 [list $v1p $v2p $v3p]
        
        lassign [::math::linearalgebra::solveGauss $matrix $bvect] u v w

        set op_projected [math::linearalgebra::scale_vect $u $v1]
        set op_projected [math::linearalgebra::add $op_projected [math::linearalgebra::scale_vect $v $v2]]
        set op_projected [math::linearalgebra::add $op_projected [math::linearalgebra::scale_vect $v $v3]]
        set op_normal [math::linearalgebra::sub_vect $op $op_projected]
        set d2 [math::linearalgebra::dotproduct $op_normal $op_normal]        
        
        set res [format "%f %f %f %f" $u $v $w $d2]
    } else {
        set res ""
    }
    return $res           
} 
    
proc HIRF::_PickPointInElementCmd { entry element_level } {    
    set xyz [GidUtils::GetCoordinates [_ "Pick point into an element"] PointInElement MESHUSE]
    if { $xyz != "" } {    
        set element_id $::GidPriv(selection) 
        set res [HIRF::GetElementRelativeCoordinatesFromCoord $element_level $xyz $element_id]
        $entry delete 0 end
        $entry insert end $res        
    }
}

#**************************************************************************************
#some Amelet plugin procedures related to conditions or materials are now moved here
# to the HIRF problemtype, to avoid the requirement of update the Amelet plugin when 
# changing the problemtype, and only require to update the HIRF problemtype for 
# next problemtype versions, but not the Amelet plugin
#
#these procedures are like GiD-events, raised by Amelet read/write, a
#nd its prototype must not be changed!!
# Amelet::ReadPreSelectorOnmeshRecursivelly { h meshpath }
# Amelet::WritePreSelectorOnmeshUnstructuredMesh { h meshpath }
# Amelet::WritePreSelectorOnmeshStructuredMesh { h meshpath }
# (h is the hdf5 file handler, meshpath is the full path to the Amelet mesh node)
#**************************************************************************************

namespace eval Amelet {
}

#**************************************************************************************
#************************************  read pre ***************************************

proc Amelet::ReadPreSelectorOnmeshRecursivelly { h meshpath } {      
    if { $meshpath == "/mesh" } {
        variable _edge_element_ids ;#to know GiD element number from its nodes
        variable _face_element_ids
        variable _volume_element_ids
        variable _node_ids
        unset -nocomplain _edge_element_ids
        unset -nocomplain _face_element_ids
        unset -nocomplain _volume_element_ids
        unset -nocomplain _node_ids
    }

    if { [$h is_group $meshpath] } {
        set childs [$h glob -directory $meshpath -types all *]
        foreach child $childs {       
            set mesh_type [Amelet::IsMesh $h $child]
            if { $mesh_type == "unstructured" } {
                Amelet::ReadPreSelectorOnmeshUnstructuredMesh $h $child           
            } elseif { $mesh_type == "structured" } {
                Amelet::ReadPreSelectorOnmeshStructuredMesh $h $child
            } else {
                Amelet::ReadPreSelectorOnmeshRecursivelly $h $child
            }               
        } 
    }   
}


#h is the hdf5 file handler
#meshpath is the full path to the Amelet mesh node
proc Amelet::ReadPreSelectorOnmeshUnstructuredMesh { h meshpath } {

    if { [Amelet::GetHirfProblemtypeVersion] >= 1.1 } {
        #read link to try to know if a selectorOnMesh must be associated to some specific condition 
        #and temporary store conditon values in array conditions_selectorOnMesh($object)
        #else if exists attribute 'type' associate
        #  type == edge -> Line_Face
        #  type == face -> Surface_Face
        #  type == pointInElement -> Line_Point_in_element or Surface_Point_in_element or Volume_Point_in_element
        
        if { [$h is_group /link] } {
            set linkgroups [$h glob -directory /link -types group *]
        } else {
            set linkgroups ""
        }
        foreach linkgroup $linkgroups {
            set link_instances [$h glob -directory $linkgroup -types group *]
            foreach link_instance $link_instances {
                set object [Amelet::GetAttribute $h $link_instance object]
                if { [$h is_dataset $object] } {
                    set subject [Amelet::GetAttribute $h $link_instance subject]                
                    if { [$h is_group $subject] } {
                        set subject_parts [split $subject /]
                        if { [lindex $subject_parts 1] == "electromagneticSource" } {
                            if { [lindex $subject_parts 2] == "generator" } {
                                set condition Line_Generator_on_face      
                                set overmesh face_elements                                
                                set questions [list ]
                                set values_default [list ]
                                foreach {question value} [lrange [GiD_Info conditions $condition] 2 end] {
                                    set index [string first #CB# $question]
                                    if { $index != -1 } {
                                        set question [string range $question 0 $index-1]
                                        #set valid_values [string range $question $index+5 end-1]
                                    }
                                    lappend questions $question
                                    lappend values_default $value
                                }
                                set values $values_default
                                set question type                        
                                set pos [lsearch -exact $questions $question]
                                if { $pos != -1 } {
                                    set value [Amelet::GetAttribute $h $subject $question]
                                    lset values $pos $value
                                } else {
                                    Amelet::Warning [_ "question '%s' not found in condition '%s'." $question $condition]                        
                                }
                                foreach group [$h glob -directory $subject -types group *] {
                                    set question [lindex [split $group /] end]                
                                    set pos [lsearch -exact $questions $question]
                                    if { $pos != -1 } {
                                        set value [Amelet::GetAttribute $h $group value]
                                        set floatingType [Amelet::GetAttribute $h $group floatingType]
                                        set physicalNature [Amelet::GetAttribute $h $group physicalNature]
                                        lset values $pos $value
                                    } else {
                                        Amelet::Warning [_ "question '%s' ignored, not found in condition '%s'." $question $condition]
                                    }                                                                                                        
                                }
                                set conditions_selectorOnMesh($object) [list $condition $overmesh $values]                            
                            } else {
                                Amelet::Warning [_ "ignored subject '%s', category '%s' not implemented." $subject [lindex $subject_parts 2]]
                            }
                        } else {
                            Amelet::Warning [_ "ignored subject '%s', category '%s' not implemented." $subject [lindex $subject_parts 1]]
                        }
                    } else {
                        Amelet::Warning [_ "Group subject '%s' not found in link '%s'." $subject $link_instance]
                    }
                } else {
                    Amelet::Warning [_ "Dataset object '%s' not found in link '%s'." $object $link_instance]                 
                }
            }
        }
    }

    if { [$h glob -types dataset ${meshpath}/elementTypes] == "" } {
        return ""
    }
    set elementcodes [lindex [$h get ${meshpath}/elementTypes] 0]
    set childs [$h glob -directory ${meshpath}/selectorOnMesh -types dataset *]    
    foreach child $childs {     
        set type [Amelet::GetAttribute $h $child type]
        set name [lindex [split $child /] end]
        if { $type == "edge" } {                        
            if { [info exists conditions_selectorOnMesh($child)] } {
                foreach {element_id amelet_face} [lindex [$h get $child] 0] {
                    set element_code [lindex $elementcodes $element_id]                                       
                    set face_id [Amelet::GiDFaceCode $element_code $amelet_face]
                    incr element_id                    
                    GiD_AssignData condition {*}$conditions_selectorOnMesh($child) [list $element_id $face_id]                   
                }                
            } else {
                set condition Line_Face                      
                foreach {element_id amelet_face} [lindex [$h get $child] 0] {
                    set element_code [lindex $elementcodes $element_id]                                       
                    set face_id [Amelet::GiDFaceCode $element_code $amelet_face]
                    incr element_id                    
                    GiD_AssignData condition $condition face_elements [list $name] [list $element_id $face_id] 
                }
            }
        } elseif { $type == "face" } {            
            set condition Surface_Face            
            foreach {element_id amelet_face} [lindex [$h get $child] 0] {
                set element_code [lindex $elementcodes $element_id]                                       
                set face_id [Amelet::GiDFaceCode $element_code $amelet_face]
                incr element_id       
                GiD_AssignData condition $condition face_elements [list $name] [list $element_id $face_id]                 
            } 
        } elseif { $type == "pointInElement" } {       
            foreach {element_id v1 v2 v3} [lindex [$h get $child] 0] {
                set element_code [lindex $elementcodes $element_id]                  
                set leveltype [Amelet::ElementEntityType $element_code]
                incr element_id        
                if { $leveltype == "edge" } {                                  
                    GiD_AssignData condition Line_Point_in_element body_elements [list $name "$v1" 0] [list $element_id]
                } elseif { $leveltype == "face" } {                  
                    GiD_AssignData condition Surface_Point_in_element body_elements [list $name "$v1 $v2" 0] [list $element_id]
                } elseif { $leveltype == "volume" } {                
                    GiD_AssignData condition Volume_Point_in_element body_elements [list $name "$v1 $v2 $v3" 0] [list $element_id]
                }
            }
        } else {
            Amelet::Warning [_ "Unexpected value '%s' of attribute 'type' for selectorOnMesh '%s'. Must be 'node' or 'element'" $type $child]          
        }        
    }       
}

proc Amelet::GetEdgeElementNumber { global_nodes_element h meshpath } {        
    variable _edge_element_ids
    variable _xs
    variable _ys
    variable _zs
    if { ![array exists _edge_element_ids] } {
        lassign [lindex [GiD_Info Mesh Elements Linear -array] 0] element_type element_ids element_connectivities element_materials
        lassign $element_connectivities nodes_0 nodes_1   
        foreach element_id $element_ids node_0 $nodes_0 node_1 $nodes_1 {
            if { $node_0 < $node_1 } {
                set key "${node_0},${node_1}"
            } else {
                set key "${node_1},${node_0}"
            }              
            set _edge_element_ids($key) $element_id
        }
        set _xs [lindex [$h get $meshpath/cartesianGrid/x] 0] 
        set _ys [lindex [$h get $meshpath/cartesianGrid/y] 0] 
        set _zs [lindex [$h get $meshpath/cartesianGrid/z] 0]
    }

    lassign $global_nodes_element node_0 node_1
    if { $node_0 < $node_1 } {
        set key "${node_0},${node_1}"
    } else {
        set key "${node_1},${node_0}"
    }
    if { [info exists _edge_element_ids($key)] } {
        set element_id $_edge_element_ids($key)
    } else {        
        foreach node_id $global_nodes_element {            
            if { [catch { GiD_Mesh get node $node_id } msg ] } {
                lassign [Amelet::GetIjkFromNum $node_id] i j k                                
                GiD_Mesh create node $node_id [list [lindex $_xs $i] [lindex $_ys $j] [lindex $_zs $k]]
            }
        }        
        set element_id [GiD_Mesh create element append linear 2 $global_nodes_element]
        set _edge_element_ids($key) $element_id
    }
    return $element_id
}

proc Amelet::GetFaceElementNumber { global_nodes_element h meshpath } {        
    variable _face_element_ids
    variable _xs
    variable _ys
    variable _zs
    if { ![array exists _face_element_ids] } {
        lassign [lindex [GiD_Info Mesh Elements Quadrilateral -array] 0] element_type element_ids element_connectivities element_materials
        lassign $element_connectivities nodes_0 nodes_1 nodes_2 nodes_3
        #get two diagonal nodes
        foreach element_id $element_ids node_0 $nodes_0 node_2 $nodes_2 {
            if { $node_0 < $node_2 } {
                set key "${node_0},${node_2}"
            } else {
                set key "${node_2},${node_0}"
            }              
            set _face_element_ids($key) $element_id
        }
        set _xs [lindex [$h get $meshpath/cartesianGrid/x] 0] 
        set _ys [lindex [$h get $meshpath/cartesianGrid/y] 0] 
        set _zs [lindex [$h get $meshpath/cartesianGrid/z] 0]
    }

    lassign $global_nodes_element node_0 node_1 node_2 node_3
    if { $node_0 < $node_2 } {
        set key "${node_0},${node_2}"
    } else {
        set key "${node_2},${node_0}"
    }
    if { [info exists _face_element_ids($key)] } {
        set element_id $_face_element_ids($key)
    } else {        
        foreach node_id $global_nodes_element {            
            if { [catch { GiD_Mesh get node $node_id } msg ] } {
                lassign [Amelet::GetIjkFromNum $node_id] i j k                                
                GiD_Mesh create node $node_id [list [lindex $_xs $i] [lindex $_ys $j] [lindex $_zs $k]]
            }
        }        
        set element_id [GiD_Mesh create element append quadrilateral 4 $global_nodes_element]
        set _face_element_ids($key) $element_id
    }
    return $element_id    
}

proc Amelet::GetVolumeElementNumber { global_nodes_element h meshpath } {        
    variable _volume_element_ids
    variable _xs
    variable _ys
    variable _zs
    if { ![array exists _volume_element_ids] } {
        lassign [lindex [GiD_Info Mesh Elements Hexahedra -array] 0] element_type element_ids element_connectivities element_materials
        lassign $element_connectivities nodes_0 nodes_1 nodes_2 nodes_3 nodes_4 nodes_5 nodes_6 nodes_7
        #get two diagonal nodes
        foreach element_id $element_ids node_0 $nodes_0 node_6 $nodes_6 {
            if { $node_0 < $node_6 } {
                set key "${node_0},${node_6}"
            } else {
                set key "${node_6},${node_0}"
            }              
            set _volume_element_ids($key) $element_id
        }
        set _xs [lindex [$h get $meshpath/cartesianGrid/x] 0] 
        set _ys [lindex [$h get $meshpath/cartesianGrid/y] 0] 
        set _zs [lindex [$h get $meshpath/cartesianGrid/z] 0]
    }

    lassign $global_nodes_element node_0 node_1 node_2 node_3 node_4 node_5 node_6 node_7
    if { $node_0 < $node_2 } {
        set key "${node_0},${node_6}"
    } else {
        set key "${node_6},${node_0}"
    }
    if { [info exists _volume_element_ids($key)] } {
        set element_id $_volume_element_ids($key)
    } else {        
        foreach node_id $global_nodes_element {            
            if { [catch { GiD_Mesh get node $node_id } msg ] } {
                lassign [Amelet::GetIjkFromNum $node_id] i j k                                
                GiD_Mesh create node $node_id [list [lindex $_xs $i] [lindex $_ys $j] [lindex $_zs $k]]
            }
        }        
        set element_id [GiD_Mesh create element append hexahedra 8 $global_nodes_element]
        set _volume_element_ids($key) $element_id
    }
    return $element_id    
}

#h is the hdf5 file handler
#meshpath is the full path to the Amelet mesh node
proc Amelet::ReadPreSelectorOnmeshStructuredMesh { h meshpath } {    
    variable _grid_n0
    variable _grid_n01
    #assume element numbers starting from 1
    if { [$h glob -types dataset ${meshpath}/cartesianGrid/x] == "" } {
        return ""
    }
    set n0 [$h get -dimensions ${meshpath}/cartesianGrid/x]
    set n1 [$h get -dimensions ${meshpath}/cartesianGrid/y]
    
    set _grid_n0 $n0
    set _grid_n01 [expr {$n0*$n1}]
    

    set childs [$h glob -directory ${meshpath}/selectorOnMesh -types dataset *]    
    foreach child $childs {     
        set type [Amelet::GetAttribute $h $child type]
        set name [lindex [split $child /] end]
        if { $type == "pointInElement" } {            
            foreach {imin jmin kmin imax jmax kmax v1 v2 v3} [lindex [$h get $child] 0] {
                if { $imin==$imax && $jmin==$jmax && $kmin==$kmax} {
                    continue ;#error, zero size cell
                }
                if { $imin==$imax } {
                    if { $jmin==$jmax || $kmin==$kmax } {
                        set leveltype edge
                    } else {
                        set leveltype face         
                    }
                } elseif { $jmin==$jmax } {                    
                    if { $kmin==$kmax || $imin==$imax } {
                        set leveltype edge
                    } else {
                        set leveltype face         
                    }
                } elseif { $kmin==$kmax } {
                    if { $imin==$imax || $jmin==$jmax } {
                        set leveltype edge
                    } else {
                        set leveltype face         
                    }                    
                } else {
                    set leveltype volume
                }
                    
                if { $leveltype == "edge" } {                               
                    set global_nodes_element [Amelet::GetSingleEdge $imin $jmin $kmin $imax $jmax $kmax] 
                    set element_id [Amelet::GetEdgeElementNumber $global_nodes_element $h $meshpath]
                    GiD_AssignData condition Line_Point_in_element body_elements [list $name "$v1" 0] $element_id                    
                } elseif { $leveltype == "face" } {    
                    set global_nodes_element [Amelet::GetSingleFace $imin $jmin $kmin $imax $jmax $kmax]                                  
                    set element_id [Amelet::GetFaceElementNumber $global_nodes_element $h $meshpath]
                    GiD_AssignData condition Surface_Point_in_element body_elements [list $name "$v1 $v2" 0] $element_id
                } elseif { $leveltype == "volume" } {   
                    set global_nodes_element [Amelet::GetSingleVolume $imin $jmin $kmin $imax $jmax $kmax]             
                    set element_id [Amelet::GetVolumeElementNumber $global_nodes_element $h $meshpath]
                    GiD_AssignData condition Volume_Point_in_element body_elements [list $name "$v1 $v2 $v3" 0] $element_id
                }
            }
        } else {
            Amelet::Warning [_ "Unexpected value '%s' of attribute 'type' for selectorOnMesh '%s'. Must be 'node' or 'element'" $type $child]          
        }        
    }       
}

#**************************************************************************************
#***************************************** write pre **********************************

#h is the hdf5 file handler
#meshpath is the full path to the Amelet mesh node
proc Amelet::WritePreSelectorOnmeshUnstructuredMesh { h meshpath } {
    variable compress_level
    variable hdf5_current_version
    
    variable NewElementNum ;#elements were renumbered for Amelet file because must be implicilty consecutive starting from 0
    variable NewNodeNum ;#nodes were renumbered for Amelet file because must be implicilty consecutive starting from 0

    if { ![Amelet::IsHirfProblemtype] } {
        
        return
    }       
   
    set point_in_element_conditions { Line_Point_in_element Surface_Point_in_element Volume_Point_in_element}
    foreach condition $point_in_element_conditions {
        set condition_data [GiD_Info conditions $condition mesh]
        if { $condition_data != "" } {
            array unset data
            foreach item $condition_data {
                set element_id [lindex $item 1]                               
                set element_id $NewElementNum($element_id)                
                set name [lindex $item 3]
                set location [lindex $item 4]
                set n [llength $location]
                if { $n == 1 } {
                    lappend location -1.0 -1.0
                } elseif { $n == 2 } {
                    lappend location -1.0
                }
                lappend data($name) $element_id {*}$location
            }
            foreach name [lsort -dictionary [array names data]] {
                set dataset_name $name
                if { [$h is_dataset ${meshpath}/selectorOnMesh/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${name}
                }
                $h set -vtype compound -fields {index uint v1 float v2 float v3 float} \
                    -compress $compress_level ${meshpath}/selectorOnMesh/$dataset_name $data($name)
                $h set_attribute -type string ${meshpath}/selectorOnMesh/$dataset_name type pointInElement
            }
        }
    }
    
    set face_element_conditions {Line_Face Surface_Face}
    set types {edge face}
    foreach condition $face_element_conditions type $types {
        set condition_data [GiD_Info conditions $condition mesh]
        if { $condition_data != "" } {        
            array unset data
            foreach item $condition_data {
                set element_id [lindex $item 0]
                set face_id [lindex $item 1]
                set name [lindex $item 3]
                set element_data [GiD_Mesh get element $element_id]
                set elemtype [lindex $element_data 1] 
                set nnodes [lindex $element_data 2]
                set amelet_faces [Amelet::EdgeOrFaceCode $elemtype $nnodes $face_id]
                set element_id $NewElementNum($element_id)
                foreach face $amelet_faces {
                    lappend data($name) $element_id $face
                }
            }
            foreach name [lsort -dictionary [array names data]] {
                set dataset_name $name
                if { [$h is_dataset ${meshpath}/selectorOnMesh/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${name}
                }
                $h set -vtype int -compress $compress_level -dimensions {0 2} ${meshpath}/selectorOnMesh/$dataset_name $data($name)
                $h set_attribute -type string ${meshpath}/selectorOnMesh/$dataset_name type $type
            }
        }
    }

    if { [Amelet::GetHirfProblemtypeVersion] >= 1.1 } {
        set condition Line_Generator_on_face
        set type edge
        set condition_data [GiD_Info conditions $condition mesh]        
        
        set questions [list ]
        set values_default [list ]
        foreach {question value} [lrange [GiD_Info conditions $condition] 2 end] {
            set index [string first #CB# $question]
            if { $index != -1 } {
                set question [string range $question 0 $index-1]
                #set valid_values [string range $question $index+5 end-1]
            }
            lappend questions $question
            lappend values_default $value
        }
        
        if { $condition_data != "" } {                    
            set sourcegroup /electromagneticSource/generator          
            $h create_group /electromagneticSource $sourcegroup

            $h create_group /link
                                                
            array unset data
            foreach item $condition_data {
                set element_id [lindex $item 0]
                set face_id [lindex $item 1]
                set name [lrange $item 3 end] ;#trick: set values in name to join all equal valued group of faces
                set element_data [GiD_Mesh get element $element_id]
                set elemtype [lindex $element_data 1] 
                set nnodes [lindex $element_data 2]
                set amelet_faces [Amelet::EdgeOrFaceCode $elemtype $nnodes $face_id]
                set element_id $NewElementNum($element_id)
                foreach face $amelet_faces {
                    lappend data($name) $element_id $face
                }
            }
            foreach name [lsort -dictionary [array names data]] {
                #selectorOnMesh
                set cont 0
                set dataset_name ${condition}_${cont}
                while { [$h is_dataset ${meshpath}/selectorOnMesh/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${cont}
                    incr cont
                }
                set selectoronmeshpath ${meshpath}/selectorOnMesh/$dataset_name
                $h set -vtype int -compress $compress_level -dimensions {0 2} $selectoronmeshpath $data($name)
                $h set_attribute -type string $selectoronmeshpath type $type
                
                #electromagneticSource
                set cont 0
                set dataset_name ${condition}_${cont}
                while { [$h is_dataset ${sourcegroup}/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${cont}
                    incr cont
                }
                set sourcepath ${sourcegroup}/$dataset_name
                $h create_group $sourcepath                

                set i 0                              
                foreach question $questions {
                    set value [lindex $name $i]
                    set value_default [lindex $values_default $i]
                    incr i
                    if { $question == "type" } {
                        #write in a special way as attribute of parent group
                        $h set_attribute -type string $sourcepath type $value                       
                    } else {                        
                        if { $value == $value_default } {
                            if  { $question != "innerImpedance" && $question != "magnitude" } {
                                #write this questions compulsory, also with default values
                                continue
                            }
                        }
                        $h create_group ${sourcepath}/$question
                        if { [llength $value] == 2 } {
                            set floatingType singleComplex
                            $h set_attribute -type string ${sourcepath}/$question floatingType $floatingType   
                            if { $hdf5_current_version >= 1.7 } {
                                $h set_attribute -type compound -fields {r float i float} ${sourcepath}/$question value $value
                            } else {
                                $h set_attribute -type string ${sourcepath}/$question value $value
                            }
                        } else {
                            set floatingType single
                            $h set_attribute -type string ${sourcepath}/$question floatingType $floatingType
                            #could implement attributes of type float, etc
                            $h set_attribute -type string ${sourcepath}/$question value $value
                        }                        
                    }
                }
                
                #for innerImpedance write also physicalNature
                $h set_attribute -type string $sourcepath/innerImpedance physicalNature impedance
                
                #link                
                set linkgroup /link/gid   
                if { ![$h is_group $linkgroup] } {
                    #create this group only if not exists
                    $h create_group $linkgroup
                }
                    
                set cont 0
                set dataset_name ${condition}_${cont}
                while { [$h is_dataset ${linkgroup}/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${cont}
                    incr cont
                }
                set linkpath ${linkgroup}/$dataset_name
                $h create_group $linkpath
                $h set_attribute -type string $linkpath object $selectoronmeshpath
                $h set_attribute -type string $linkpath subject $sourcepath
            }                    
        }        
    }
}

proc Amelet::WritePreSelectorOnmeshStructuredMesh { h meshpath } {
    variable compress_level    
           
    set point_in_element_conditions { Line_Point_in_element Surface_Point_in_element Volume_Point_in_element }
    set entity_types { edge face volume }
    foreach condition $point_in_element_conditions entityType $entity_types {
        set condition_data [GiD_Info conditions $condition mesh]
        if { $condition_data != "" } {
            array unset data           
            foreach item $condition_data {
                set element_id [lindex $item 1]
                set idnodes [lrange [GiD_Mesh get element $element_id] 3 end]
                if { $entityType == "volume" } {                    
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    #lassign $ijk0 i j k
                    #lappend elements $i $j $k [expr $i+1] [expr $j+1] [expr $k+1] ;#assumed first node as lower left corner!!
                    set ijk_6 [Amelet::GetIjkFromNum [lindex $idnodes 6]]
                    set element [list {*}$ijk_0 {*}$ijk_6]
                } elseif { $entityType == "face" } {                                           
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    set ijk_2 [Amelet::GetIjkFromNum [lindex $idnodes 2]]                           
                    set element [list {*}$ijk_0 {*}$ijk_2]
                } elseif { $entityType == "edge" } {                                           
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    set ijk_1 [Amelet::GetIjkFromNum [lindex $idnodes 1]]
                    set element [list {*}$ijk_0 {*}$ijk_1]
                } else {
                    #entity not valid for Amelet structured (cartesian) mesh
                    continue 
                }               

                set name [lindex $item 3]
                set location [lindex $item 4]
                set n [llength $location]
                if { $n == 1 } {
                    lappend location -1.0 -1.0
                } elseif { $n == 2 } {
                    lappend location -1.0
                }
                lappend data($name) {*}$element {*}$location
            }

            foreach name [lsort -dictionary [array names data]] {
                set dataset_name $name
                if { [$h is_dataset ${meshpath}/selectorOnMesh/$dataset_name] } {
                    #if the same name is used for other condition add a prefix to have a different name
                    set dataset_name ${condition}_${name}
                }
                $h set -vtype compound -fields {imin uint jmin uint kmin uint imax uint jmax uint kmax uint v1 float v2 float v3 float} \
                    -compress $compress_level ${meshpath}/selectorOnMesh/$dataset_name $data($name)
                $h set_attribute -type string ${meshpath}/selectorOnMesh/$dataset_name type pointInElement
            }
        }
    }
    
    #don't write selectorOnMesh of types edge or face for structured meshes, 
    #as is docummented in Amelet format (suggested to use groups for this selection)
}
